@@ -1,9 +1,18 @@
 //! parsing server response
-use crate::error::{Error, ParseErr};
-use std::{collections::HashMap, fmt, io::Write, str};
+use crate::{
+    error::{Error, ParseErr},
+    uri::Uri,
+};
+use std::{
+    collections::{hash_map, HashMap},
+    fmt,
+    io::Write,
+    str,
+};
 
 pub(crate) const CR_LF_2: [u8; 4] = [13, 10, 13, 10];
 
+#[derive(Debug, PartialEq, Clone)]
 pub struct Response {
     status: Status,
     headers: Headers,
@@ -20,26 +29,26 @@ impl Response {
     ///Parses `Response` from slice of bytes. Writes it's body to `writer`.
     pub fn try_from<T: Write>(res: &[u8], writer: &mut T) -> Result<Response, Error> {
         if res.is_empty() {
-            return Err(Error::Parse(ParseErr::Empty));
+            Err(Error::Parse(ParseErr::Empty))
+        } else {
+            let mut pos = res.len();
+            if let Some(v) = find_slice(res, &CR_LF_2) {
+                pos = v;
+            }
+
+            let response = Self::from_head(&res[..pos])?;
+            writer.write_all(&res[pos..])?;
+
+            Ok(response)
         }
-
-        let mut pos = res.len();
-        if let Some(v) = find_slice(res, &CR_LF_2) {
-            pos = v;
-        }
-
-        let response = Self::from_head(&res[..pos])?;
-        writer.write_all(&res[pos..])?;
-
-        Ok(response)
     }
 
     ///Parses head of a `Response` - status and headers - from slice of bytes.
     pub fn parse_head(head: &[u8]) -> Result<(Status, Headers), ParseErr> {
         let mut head = str::from_utf8(head)?.splitn(2, '\n');
 
-        let status = head.next().unwrap().parse()?;
-        let headers = head.next().unwrap().parse()?;
+        let status = head.next().ok_or(ParseErr::Invalid)?.parse()?;
+        let headers = head.next().ok_or(ParseErr::Invalid)?.parse()?;
 
         Ok((status, headers))
     }
@@ -74,6 +83,134 @@ impl Response {
     }
 }
 
+#[derive(PartialEq, Debug, Clone)]
+pub struct Status {
+    version: String,
+    code: StatusCode,
+    reason: String,
+}
+
+impl<T, U, V> From<(T, U, V)> for Status
+where
+    T: ToString,
+    V: ToString,
+    StatusCode: From<U>,
+{
+    fn from(status: (T, U, V)) -> Status {
+        Status {
+            version: status.0.to_string(),
+            code: StatusCode::from(status.1),
+            reason: status.2.to_string(),
+        }
+    }
+}
+
+impl str::FromStr for Status {
+    type Err = ParseErr;
+
+    fn from_str(status_line: &str) -> Result<Status, Self::Err> {
+        let mut status_line = status_line.trim().splitn(3, ' ');
+
+        let version = status_line.next().ok_or(ParseErr::Invalid)?;
+        let code: u16 = status_line.next().ok_or(ParseErr::Invalid)?.parse()?;
+        let reason = status_line.next().ok_or(ParseErr::Invalid)?;
+
+        Ok(Status::from((version, code, reason)))
+    }
+}
+
+#[derive(Debug, PartialEq, Clone, Default)]
+///Wrapper around HashMap<String, String> with additional functionality for parsing HTTP headers
+pub struct Headers(HashMap<String, String>);
+
+impl Headers {
+    ///Creates an empty `Headers`.
+    ///
+    ///The headers are initially created with a capacity of 0, so they will not allocate until
+    ///it is first inserted into.
+    pub fn new() -> Headers {
+        Headers(HashMap::new())
+    }
+
+    ///Creates empty `Headers` with the specified capacity.
+    ///
+    ///The headers will be able to hold at least capacity elements without reallocating.
+    ///If capacity is 0, the headers will not allocate.
+    pub fn with_capacity(capacity: usize) -> Headers {
+        Headers(HashMap::with_capacity(capacity))
+    }
+
+    ///An iterator visiting all key-value pairs in arbitrary order.
+    ///The iterator element type is (&String, &String).
+    pub fn iter(&self) -> hash_map::Iter<String, String> {
+        self.0.iter()
+    }
+
+    ///Returns a reference to the value corresponding to the key.
+    pub fn get(&self, v: &str) -> Option<&std::string::String> {
+        self.0.get(v)
+    }
+
+    ///Inserts a key-value pair into the headers.
+    ///
+    ///If the headers did not have this key present, None is returned.
+    ///
+    ///If the headers did have this key present, the value is updated, and the old value is returned.
+    ///The key is not updated, though; this matters for types that can be == without being identical.
+    pub fn insert<T, U>(&mut self, key: &T, val: &U) -> Option<String>
+    where
+        T: ToString + ?Sized,
+        U: ToString + ?Sized,
+    {
+        self.0.insert(key.to_string(), val.to_string())
+    }
+
+    ///Creates default headers for a HTTP request
+    pub fn default_http(uri: &Uri) -> Headers {
+        let mut headers = Headers::with_capacity(4);
+
+        headers.insert("Host", uri.host().unwrap_or(""));
+        headers.insert("Referer", uri);
+
+        headers
+    }
+}
+
+impl str::FromStr for Headers {
+    type Err = ParseErr;
+
+    fn from_str(s: &str) -> Result<Headers, ParseErr> {
+        let headers = s.trim();
+
+        if headers.lines().all(|e| e.contains(':')) {
+            let headers = headers
+                .lines()
+                .map(|elem| {
+                    let idx = elem.find(": ").unwrap();
+                    let (key, value) = elem.split_at(idx);
+                    (key.to_string(), value[2..].to_string())
+                })
+                .collect();
+
+            Ok(Headers(headers))
+        } else {
+            Err(ParseErr::Invalid)
+        }
+    }
+}
+
+impl From<HashMap<String, String>> for Headers {
+    fn from(map: HashMap<String, String>) -> Headers {
+        Headers(map)
+    }
+}
+
+impl From<Headers> for HashMap<String, String> {
+    fn from(map: Headers) -> HashMap<String, String> {
+        map.0
+    }
+}
+
 ///Code sent by a server in response to a client's request.
 ///# Example
 ///```
@@ -142,88 +279,6 @@ impl fmt::Display for StatusCode {
     }
 }
 
-#[derive(PartialEq, Debug)]
-pub struct Status {
-    version: String,
-    code: StatusCode,
-    reason: String,
-}
-
-impl<T, U, V> From<(T, U, V)> for Status
-where
-    T: ToString,
-    V: ToString,
-    StatusCode: From<U>,
-{
-    fn from(status: (T, U, V)) -> Status {
-        Status {
-            version: status.0.to_string(),
-            code: StatusCode::from(status.1),
-            reason: status.2.to_string(),
-        }
-    }
-}
-
-impl str::FromStr for Status {
-    type Err = ParseErr;
-
-    fn from_str(status_line: &str) -> Result<Status, ParseErr> {
-        let status_line: Vec<_> = status_line.trim().splitn(3, ' ').collect();
-
-        let version = status_line[0];
-        let code: u16 = status_line[1].parse()?;
-        let reason = status_line[2];
-
-        Ok(Status::from((version, code, reason)))
-    }
-}
-
-#[derive(Debug, PartialEq)]
-pub struct Headers(HashMap<String, String>);
-
-impl Headers {
-    ///Returns a reference to the value corresponding to the key.
-    pub fn get(&self, v: &str) -> Option<&std::string::String> {
-        self.0.get(v)
-    }
-}
-
-impl str::FromStr for Headers {
-    type Err = ParseErr;
-
-    fn from_str(s: &str) -> Result<Headers, ParseErr> {
-        let headers: Vec<_> = s.trim().lines().collect();
-        let correct_headers = headers.iter().all(|e| e.contains(':'));
-
-        if correct_headers {
-            let headers = headers
-                .iter()
-                .map(|elem| {
-                    let pos = elem.find(':').unwrap();
-                    let (key, value) = elem.split_at(pos);
-                    (key.to_string(), value[2..].to_string())
-                })
-                .collect();
-
-            Ok(Headers(headers))
-        } else {
-            Err(ParseErr::Invalid)
-        }
-    }
-}
-
-impl From<HashMap<String, String>> for Headers {
-    fn from(map: HashMap<String, String>) -> Headers {
-        Headers(map)
-    }
-}
-
-impl From<Headers> for HashMap<String, String> {
-    fn from(map: Headers) -> HashMap<String, String> {
-        map.0
-    }
-}
-
 ///Finds elements slice `e` inside slice `data`. Returns position of the end of first match.
 pub fn find_slice<T>(data: &[T], e: &[T]) -> Option<usize>
 where
@@ -372,14 +427,14 @@ mod tests {
     }
 
     #[test]
-    fn headers_get() {
-        let mut headers = HashMap::with_capacity(2);
-        headers.insert(
-            "Date".to_string(),
-            "Sat, 11 Jan 2003 02:44:04 GMT".to_string(),
-        );
+    fn headers_new() {
+        assert_eq!(Headers::new(), Headers(HashMap::new()));
+    }
 
-        let headers = Headers::from(headers);
+    #[test]
+    fn headers_get() {
+        let mut headers = Headers::with_capacity(2);
+        headers.insert("Date", "Sat, 11 Jan 2003 02:44:04 GMT");
 
         assert_eq!(
             headers.get("Date"),
@@ -387,6 +442,31 @@ mod tests {
         );
     }
 
+    #[test]
+    fn headers_insert() {
+        let mut headers_expect = HashMap::new();
+        headers_expect.insert("Connection".to_string(), "Close".to_string());
+        let headers_expect = Headers(headers_expect);
+
+        let mut headers = Headers::new();
+        headers.insert("Connection", "Close");
+
+        assert_eq!(headers_expect, headers);
+    }
+
+    #[test]
+    fn headers_default_http() {
+        let uri = "http://doc.rust-lang.org/std/string/index.html"
+            .parse()
+            .unwrap();
+
+        let mut headers = Headers::with_capacity(4);
+        headers.insert("Host", "doc.rust-lang.org");
+        headers.insert("Referer", "http://doc.rust-lang.org/std/string/index.html");
+
+        assert_eq!(Headers::default_http(&uri), headers);
+    }
+
     #[test]
     fn headers_from_str() {
         let mut headers_expect = HashMap::with_capacity(2);
@@ -403,7 +483,7 @@ mod tests {
 
     #[test]
     fn headers_from() {
-        let mut headers_expect = HashMap::with_capacity(2);
+        let mut headers_expect = HashMap::with_capacity(4);
         headers_expect.insert(
             "Date".to_string(),
             "Sat, 11 Jan 2003 02:44:04 GMT".to_string(),
@@ -418,7 +498,25 @@ mod tests {
     }
 
     #[test]
-    fn elements_find() {
+    fn hash_map_from_headers() {
+        let mut headers = Headers::with_capacity(4);
+        headers.insert("Date", "Sat, 11 Jan 2003 02:44:04 GMT");
+        headers.insert("Content-Type", "text/html");
+        headers.insert("Content-Length", "100");
+
+        let mut headers_expect = HashMap::with_capacity(4);
+        headers_expect.insert(
+            "Date".to_string(),
+            "Sat, 11 Jan 2003 02:44:04 GMT".to_string(),
+        );
+        headers_expect.insert("Content-Type".to_string(), "text/html".to_string());
+        headers_expect.insert("Content-Length".to_string(), "100".to_string());
+
+        assert_eq!(HashMap::from(headers), headers_expect);
+    }
+
+    #[test]
+    fn find_slice_e() {
         const WORDS: [&str; 8] = ["Good", "job", "Great", "work", "Have", "fun", "See", "you"];
         const SEARCH: [&str; 3] = ["Great", "work", "Have"];
 
@@ -447,13 +545,10 @@ mod tests {
 
     #[test]
     fn res_parse_head() {
-        let mut headers = HashMap::with_capacity(2);
-        headers.insert(
-            "Date".to_string(),
-            "Sat, 11 Jan 2003 02:44:04 GMT".to_string(),
-        );
-        headers.insert("Content-Type".to_string(), "text/html".to_string());
-        headers.insert("Content-Length".to_string(), "100".to_string());
+        let mut headers = Headers::with_capacity(4);
+        headers.insert("Date", "Sat, 11 Jan 2003 02:44:04 GMT");
+        headers.insert("Content-Type", "text/html");
+        headers.insert("Content-Length", "100");
 
         let head = Response::parse_head(RESPONSE_H).unwrap();
 
@@ -490,13 +585,10 @@ mod tests {
         let mut writer = Vec::new();
         let res = Response::try_from(RESPONSE, &mut writer).unwrap();
 
-        let mut headers = HashMap::with_capacity(2);
-        headers.insert(
-            "Date".to_string(),
-            "Sat, 11 Jan 2003 02:44:04 GMT".to_string(),
-        );
-        headers.insert("Content-Type".to_string(), "text/html".to_string());
-        headers.insert("Content-Length".to_string(), "100".to_string());
+        let mut headers = Headers::with_capacity(2);
+        headers.insert("Date", "Sat, 11 Jan 2003 02:44:04 GMT");
+        headers.insert("Content-Type", "text/html");
+        headers.insert("Content-Length", "100");
 
         assert_eq!(res.headers(), &Headers::from(headers));
     }
